import json

from src.lunar_policy import Check, ComponentData


class TestCheckExists:
    def test_exists_valid_path_workflows_finished(self):
        data = ComponentData.from_component_json(json.dumps({
            "hi": "there"
        }), bundle_info={"workflows_finished": True})

        with Check("test", data=data) as c:
            assert c.exists(".hi")

    def test_exists_valid_path_workflows_not_finished(self):
        data = ComponentData.from_component_json(json.dumps({
            "hi": "there"
        }), bundle_info={"workflows_finished": False})

        with Check("test", data=data) as c:
            assert c.exists(".hi")

    def test_exists_missing_path_workflows_finished(self):
        data = ComponentData.from_component_json(
            "{}",
            bundle_info={"workflows_finished": True}
        )

        with Check("test", data=data) as c:
            assert not c.exists(".missing")

    def test_exists_missing_path_workflows_not_finished(self, capsys):
        data = ComponentData.from_component_json(
            "{}",
            bundle_info={"workflows_finished": False}
        )

        with Check("test", data=data) as c:
            c.exists(".missing")
            c.assert_equals("should not run", "should not run")

        captured = capsys.readouterr()
        results = json.loads(captured.out)["assertions"]
        assert len(results) == 1
        assert results[0]["result"] == "no-data"
        assert ".missing" in results[0]["failure_message"]
        assert results[0]["op"] == "fail"



import json
import pytest

from src.lunar_policy import Check, ComponentData


class TestCheckGet:
    def test_get_valid_path_workflows_finished(self):
        data = ComponentData.from_component_json(json.dumps({
            "hi": "there"
        }), bundle_info={"workflows_finished": True})

        with Check("test", data=data) as c:
            c.get(".hi")

    def test_get_valid_path_workflows_not_finished(self):
        data = ComponentData.from_component_json(json.dumps({
            "hi": "there"
        }), bundle_info={"workflows_finished": False})

        with Check("test", data=data) as c:
            c.get(".hi")

    def test_get_missing_path_workflows_finished(self):
        data = ComponentData.from_component_json(
            "{}",
            bundle_info={"workflows_finished": True}
        )

        with pytest.raises(ValueError):
            with Check("test", data=data) as c:
                c.get(".missing")

    def test_get_missing_path_workflows_not_finished(self, capsys):
        data = ComponentData.from_component_json(
            "{}",
            bundle_info={"workflows_finished": False}
        )

        with Check("test", data=data) as c:
            c.get(".missing")

        captured = capsys.readouterr()
        results = json.loads(captured.out)["assertions"]
        assert len(results) == 1
        assert results[0]["result"] == "no-data"
        assert ".missing" in results[0]["failure_message"]
        assert results[0]["op"] == "fail"

    def test_get_invalid_path(self):
        data = ComponentData.from_component_json("{}")

        with pytest.raises(ValueError):
            with Check("test", data=data) as c:
                c.get("@#$@#$@#$")

    def test_get_all_valid_path_single(self):
        data = ComponentData.from_json(json.dumps({
            "bundle_info": {},
            "merged_blob": {},
            "metadata_instances": [
                {
                    "payload": {
                        "hi": "there"
                    }
                }
            ]
        }))

        all = []
        with Check("test", data=data) as c:
            all = c.get_all(".hi")

        assert all == ["there"]

    def test_get_all_valid_path_multiple(self):
        data = ComponentData.from_json(json.dumps({
            "bundle_info": {},
            "merged_blob": {},
            "metadata_instances": [
                {
                    "payload": {
                        "hi": "there"
                    }
                },
                {
                    "payload": {
                        "hi": "there"
                    }
                }
            ]
        }))

        all = []
        with Check("test", data=data) as c:
            all = c.get_all(".hi")

        assert all == ["there", "there"]

    def test_get_all_invalid_path(self):
        data = ComponentData.from_component_json("{}")

        with pytest.raises(ValueError):
            with Check("test", data=data) as c:
                c.get_all("@#$@#$@#$")



import json

from src.lunar_policy import Check, ComponentData


class TestCheckGet:
    def test_one_check_output_with_prefix(self, capsys, monkeypatch):
        data = ComponentData.from_component_json(json.dumps({
            "hi": "there"}))
        prefix = "LUNAR_SNIPPETS: "

        monkeypatch.setenv("LUNAR_LOG_PREFIX", prefix)

        with Check("test", data=data) as c:
            c.get(".hi")

        captured = capsys.readouterr()
        assert captured.out.startswith(prefix), f"Missing prefix: {captured}"

        json_part = captured.out[len(prefix):]
        try:
            json.loads(json_part)
        except Exception as e:
            raise AssertionError(f"Invalid JSON: {json_part!r}") from e

    def test_multiple_checks_output_with_prefix(self, capsys, monkeypatch):
        data = ComponentData.from_component_json(json.dumps({
            "hi": "there"}))
        prefix = "LUNAR_SNIPPETS: "

        monkeypatch.setenv("LUNAR_LOG_PREFIX", prefix)

        with Check("test", data=data) as c:
            c.get(".hi")

        with Check("test", data=data) as c:
            c.get(".hi")

        captured = capsys.readouterr()
        lines = captured.out.splitlines()
        assert len(lines) == 2, f"Expected 2 lines, got {len(lines)}"

        for line in captured.out.splitlines():
            assert line.startswith(prefix), f"Missing prefix: {line!r}"
            json_part = line[len(prefix):]
            try:
                json.loads(json_part)
            except Exception as e:
                raise AssertionError(f"Invalid JSON: {json_part!r}") from e
